<?php
namespace Ep\View\Renderer;

use Ak\ServiceManager\AbstractFactory\PluginManagerAbstractFactory;
use Ak\View\HelperPluginManager;
use Ep\View\Helper\TwigBridge;
use Interop\Container\ContainerInterface;
use Twig\Environment;
use Twig\Error\LoaderError;
use Twig\Extension\DebugExtension;
use Twig\Loader\FilesystemLoader;
use Twig\Loader\LoaderInterface;
use Twig\TwigFunction;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Factory\FactoryInterface;
use Laminas\View\Renderer\RendererInterface;

/**
 * Stvara novu instancu TwigRenderera.
 *
 * @package Ep\View\Renderer
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class TwigRendererFactory implements FactoryInterface
{
    /**
     * Stvara renderer.
     *
     * @param ContainerInterface $container
     * @param string $requestedName
     * @param null|array $options
     *
     * @return RendererInterface
     * @throws LoaderError
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $options ?? $container->get('config')['twig'] ?? [];

        if ($container->get('config')['app']['development'] ?? false) {
            $config['options']['debug'] = true;
            $config['options']['auto_reload'] = true;
        } else {
            if (!is_dir('var/cache/twig')) {
                mkdir('var/cache/twig', 0777, true);
            }

            $config['options']['cache'] = realpath('var/cache/twig');
        }

        $twig = $this->createTwig($container, $config);
        $renderer = new TwigRenderer($twig);

        $helperPluginManagerFactory = new PluginManagerAbstractFactory();
        /** @var HelperPluginManager $helperPluginManager */
        $helperPluginManager = $helperPluginManagerFactory($container, HelperPluginManager::class);

        $renderer->setHelperPluginManager($helperPluginManager);

        $twig->addFunction(new TwigFunction('*', $helperPluginManager->get(TwigBridge::class), [
            'is_safe' => ['all'],
        ]));

        return $renderer;
    }

    /**
     * Stvara novi TwigLoader.
     *
     * @param array $config
     *
     * @return LoaderInterface
     * @throws LoaderError
     */
    private function createLoader(array $config): LoaderInterface
    {
        if (!isset($config['template_paths']) || empty($config['template_paths'])) {
            throw new ServiceNotCreatedException("Polje 'template_paths' je obavezno.");
        }

        if (!is_array($config['template_paths'])) {
            throw new ServiceNotCreatedException("Polje 'template_paths' mora biti array.");
        }

        $loader = new FilesystemLoader();

        foreach ($config['template_paths'] as $namespace => $path) {
            if (!is_array($path) && !is_dir($path)) {
                throw new ServiceNotCreatedException(sprintf(
                    "Nepostojeća putanja '%s'.", $path
                ));
            }

            $namespace = is_numeric($namespace) ? FilesystemLoader::MAIN_NAMESPACE : $namespace;

            if (is_array($path)) {
                foreach ($path as $p) {
                    if (!is_dir($p)) {
                        throw new ServiceNotCreatedException(sprintf(
                            "Nepostojeća putanja '%s'.", $path
                        ));
                    }

                    $loader->prependPath($p, $namespace);
                }
            } else {
                $loader->prependPath($path, $namespace);
            }
        }

        return $loader;
    }

    /**
     * @param ContainerInterface $container
     * @param array $config
     *
     * @return Environment
     * @throws LoaderError
     */
    private function createTwig(ContainerInterface $container, array $config): Environment
    {
        $loader = $this->createLoader($config);
        $twig = new Environment($loader, $config['options'] ?? []);

        foreach (array_unique($config['extensions'] ?? []) as $extension) {
            $twig->addExtension($container->get($extension));
        }

        if ($config['options']['debug'] ?? false) {
            $twig->addExtension(new DebugExtension());
        }

        return $twig;
    }
}
